/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "pokerui.h"

void pokerUiInit(pokerui_t *ui) {
  uint8_t i;

  // Initialize card render(s)
  imageInit(&ui->card);

  // Initialize the player face quad.
  quadInit(&ui->quad, 0,
    0, 0, 0, 1,
    POKER_UI_PLAYER_IMAGE_SIZE, POKER_UI_PLAYER_IMAGE_SIZE, 1, 0
  );

  // Initialize the grid
  gridInit(&ui->grid);
  
  ui->grid.gutterX = POKER_UI_PLAYER_PADDING;
  ui->grid.columns = 2;
  ui->grid.rows = 2;
  ui->grid.columnDefinitions[1] = POKER_UI_PLAYER_IMAGE_SIZE;
  ui->grid.rowDefinitions[0] = POKER_UI_PLAYER_IMAGE_SIZE / 2.0f;
  ui->grid.rowDefinitions[1] = POKER_UI_PLAYER_IMAGE_SIZE / 2.0f;

  gridResize(&ui->grid, POKER_UI_PLAYER_WIDTH, POKER_UI_PLAYER_HEIGHT);

  // Initialize the label
  labelInit(&ui->label);
  
  // Initialize the frames for each player.
  for(i = 0; i < POKER_PLAYER_COUNT; i++) {
    frameBufferInit(ui->frames + i,
      POKER_UI_PLAYER_IMAGE_RESOLUTION,
      POKER_UI_PLAYER_IMAGE_RESOLUTION
    );
  }
}

void pokerUiUpdate(
  pokerui_t *ui, engine_t *engine, shader_t *shader,
  vncharacter_t *characters, pokerplayer_t *players
) {
  uint8_t i, j;
  camera_t camera;
  uint8_t seat;
  float x, y, z;
  pokerplayer_t *player;

  // Set up the camera perspective
  cameraPerspective(&camera, 45,
    (float)POKER_UI_PLAYER_IMAGE_SIZE / (float)POKER_UI_PLAYER_IMAGE_SIZE,
    0.03f, 10.0f
  );

  // Render the face of each player.
  j = 0;
  for(i = 0; i < POKER_PLAYER_COUNT; i++) {
    if(i == POKER_PLAYER_HUMAN_INDEX) continue;
    player = players + j;

    // Locate the XYZ position of the camera to look at the player
    seat = pokerGameSeatFromIndex(i);
    x = POKER_WORLD_SEAT_POSITION_X(seat);
    y = POKER_UI_PLAYER_IMAGE_Y;
    z = POKER_WORLD_SEAT_POSITION_Z(seat);

    cameraLookAt(&camera,
      x * POKER_UI_PLAYER_IMAGE_DIST, y, z * POKER_UI_PLAYER_IMAGE_DIST,
      x, y, z
    );
    
    // Bind the frame buffer
    frameBufferUse(&ui->frames + j, true);

    // Render the VN character
    shaderUse(shader);
    shaderUseCamera(shader, &camera);
    shaderUsePosition(shader, 0,0,0, 0,0,0);
    vnCharacterRender(characters + playerIndex, shader);

    // Increment
    j++;
  }

  renderResetFramebuffer(&engine->render);
}

void pokerUiRender(pokerui_t *ui, engine_t *engine, shader_t *shader) {
  uint8_t i, j;
  pokerplayerui_t *ui;
  pokerplayer_t *player;
  char message[128];
  float scale;
  align_t align;
  float gx, gy, gw, gh, x, y;

  // Get the default font scale size.
  scale = fontGetScale(FONT_SIZE_DEFAULT);

  // <testing>
  if(pokerGame->poker.state >= POKER_STATE_DEALING) {
    for(j = 0; j < POKER_PLAYER_COUNT; j++) {
      player = pokerGame->poker.players + j;
      for(i = 0; i < player->cardCount; i++) {
        pokerCardSetImage(&pokerGame->ui.card, &pokerGame->assets.cardTexture, player->cards[i]);
        imageRender(&pokerGame->ui.card, &pokerGame->assets.shader, i * 64.0f, j * 100);
      }
    }
  }

  player = pokerGame->poker.players + POKER_PLAYER_HUMAN_INDEX;
  if(pokerGame->poker.bet.better == POKER_PLAYER_HUMAN_INDEX) {
    sprintf(message, "Press down to fold, up to bet, right to check/call.");
    labelSetText(&pokerGame->ui.player->label, &pokerGame->assets.font, message);
    labelRender(&pokerGame->ui.player->label, &pokerGame->assets.shader, 300, 100);
  }
  // </testing>
  j = 0;
  for(i = 0; i < POKER_PLAYER_COUNT; i++) {
    if(i == POKER_PLAYER_HUMAN_INDEX) continue;

    // Get the player.
    player = game->poker.players + i;

    // Position the grid itself.
    x = 0;
    y = POKER_UI_PLAYER_HEIGHT * j;

    // Render face
    gridGetChild(&ui->grid, 1, 0, 1, 2, &gx, &gy, &gw, &gh);
    shaderUseTexture(shader, &ui->frames + j);
    shaderUsePosition(shader, x + gx, y + gy, 0, 0,0,0);
    primitiveDraw(&ui->quad, 0, -1);

    // Render chips
    sprintf(buffer, "$%i", player->chips);
    ui->label.maxWidth = -1;
    labelSetText(&ui->label, font, buffer);
    align = gridGetAndAlignChild(
      &ui->grid, 0, 0, 1, 1, 
      ALIGN_POS_END|ALIGN_SIZE_ORIGINAL, ALIGN_POS_CENTER|ALIGN_SIZE_ORIGINAL,
      ui->label.info.width, ui->label.info.height
    );
    labelRender(&ui->label, shader, x+align.x, y+align.y);
    
    // Render state
    if(player->state & POKER_PLAYER_STATE_OUT) {
      sprintf(buffer, "Out");
    } else if(player->state & POKER_PLAYER_STATE_FOLDED) {
      sprintf(buffer, "Folded");
    } else if(player->state & POKER_PLAYER_STATE_SHOWING) {
      sprintf(buffer, "Showing");
    } else if(game->poker.bet.better == playerIndex) {
      sprintf(buffer, "Thinking");
    } else {
      sprintf(buffer, "Whatever");
    }
    labelSetText(&ui->label, font, buffer);
    align = gridGetAndAlignChild(
      &ui->grid, 0, 1, 1, 1, 
      ALIGN_POS_END | ALIGN_SIZE_ORIGINAL, ALIGN_POS_CENTER | ALIGN_SIZE_ORIGINAL,
      ui->label.info.width, ui->label.info.height
    );
    labelRender(&ui->label, shader, x+align.x, y+align.y);

    // Increment.
    j++;
  }
}

void pokerUiDispose(pokerui_t *ui) {
  uint8_t i;

  for(i = 0; i < POKER_PLAYER_COUNT; i++) frameBufferDispose(ui->frames + i);
  labelDispose(&ui->label);
  primitiveDispose(&ui->quad);
  imageDispose(&ui->card);
}

void pokerUiSetImageToCard(image_t *image, texture_t *texture, card_t card) {
  uint8_t cardImageIndex = (
    cardGetNumber(card) == CARD_ACE ? (
      card - CARD_COUNT_PER_SUIT + 1
    ) : card+0x01
  );

  imageSetTextureAndCrop(
    image, texture,
    cardGetNumber(cardImageIndex) * 71,
    cardGetSuit(card) * 96,
    71, 96
  );
}